@@ -1,21 +1,25 @@
 /*
 
- wmcube.c 
- Version 0.98 (2000-10-20)
+ wmcube.c
 
- Robert Kling (robkli-8@student.luth.se)
+ Version 1.0.0  (2014-11-26)
+ Cezary M. Kruk <c.kruk@bigfoot.com>
+ http://linux-bsd-unix.strefa.pl
+
+ Versions 0.98  (2000-10-23)
+ Robert Kling   <robkli-8@student.luth.se>
  http://boombox.campus.luth.se/projects.php
 
  Contributions:
- 	-n option patch by Thorsten Jens (thodi@et-inf.fho-emden.de) (2000-05-12)
- 	Various bugfixes and optimizations by Jakob Borg (2000-05-13)
+	-n option patch by Thorsten Jens (thodi@et-inf.fho-emden.de) (2000-05-12)
+	Various bugfixes and optimizations by Jakob Borg (2000-05-13)
 	Solaris Port by Dan Price (dp@rampant.org) (2000-07-16)
 	OpenBSD Port by Brian Joseph Czapiga (rys@godsey.net) (2000-07-19)
 	FreeBSD Port by Tai-hwa Liang (avatar@mmlab.cse.yzu.edu.tw) (2000-07-20)
 	NetBSD Port by Jared Smolens <jsmolens+@andrew.cmu.edu> (2000-09-23)
 
  This software is licensed through the GNU General Public Licence.
- 
+
  See http://www.BenSinclair.com/dockapp/ for more wm dock apps.
 
  If you want to port wmcube to another OS the system specific code is
@@ -23,8 +27,12 @@
 
 */
 
-#define WMCUBE_VERSION "0.98"
-#define REV_DATE "2000-10-23"
+#define WMCUBE_VERSION "1.0.0"
+#define REV_YEAR "2014"
+#define REV_DATE "2014-11-26"
+#define WMCUBE_OLDVERSION "0.98"
+#define OLDREV_YEAR "2000"
+#define OLDREV_DATE "2000-10-23"
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -57,6 +65,12 @@
 #include "../wmgeneral/misc.h"
 
 #include "wmcube.xpm"
+#include "wmcubered.xpm"
+#include "wmcubegreen.xpm"
+#include "wmcubeblue.xpm"
+#include "wmcubecyan.xpm"
+#include "wmcubemagenta.xpm"
+#include "wmcubeyellow.xpm"
 char wmcube_mask_bits[64*64];
 int  wmcube_mask_width = 64;
 int  wmcube_mask_height = 64;
@@ -75,7 +89,11 @@ void BlitString(char *name, int x, int y);
 void BlitNum(int num, int x, int y);
 void clearscr();
 void draw();
+void set_plugin();
 void startup_seq();
+int  red = 0;
+int  green = 0;
+int  blue = 0;
 
 //**** 3d specific ********************************
 
@@ -102,23 +120,26 @@ int  calc_cpu_total();
 
 //**** Global variables ***************************
 
-int 	xcenter, ycenter, zoff;
-double	cost[361], sint[361];
-double  acost[100];
-float 	**matrix;
-float	**rmatrix;
+int		xcenter, ycenter, zoff;
+double		cost[361], sint[361];
+double		acost[100];
+float		**matrix;
+float		**rmatrix;
 int		**planes;
 int		*plane_color;
 int		*zorder;
 int		*cline;
 int		nofcoords, noflines, nofplanes;
-char	*objects[1000];
+char		*objects[1000];
 int		nof_objects = 0;
 int		show_load = 1;
 int		use_nice = 1;
 int		which_cpu = -1;
 int		planesORlines = 1;
-char 	*pname;
+char		*pname;
+int		font = 0;
+
+int		but_stat = -1;
 
 float	lum_vector[3] = { 0, 0, 100 };  // Lightsource vector
 
@@ -127,18 +148,19 @@ static kvm_t            *kd;
 static struct nlist     nlst[] = { {"_cp_time"}, {0} };
 #endif
 
+	char	obj_filename[256];
+	char	*plugin = {""};
+
 int main(int argc, char **argv)
 {
 	int		j, i = 0, rot_speed = 0, cpu_usage = 0, rot_step = 1;
 	long	screen_speed = 10000;    // microseconds between screen updates (approx.)
 	long	cpu_update = 490000;     // microseconds between cpu update (approx.)
-	int		but_stat = -1;
 	int		loop = 0;
 	XEvent	Event;
 
 	char	*rotdiv = {"25"};
 	char	*rotstep = {"1"};
-	char	*obj_filename = {""};
 	int		rot;
 	int 	cube_color = 1;
 	int 	c = 0;
@@ -150,36 +172,48 @@ int main(int argc, char **argv)
 	srand((unsigned)time(NULL));
 	opterr = 0;
 	
-	while ((c = getopt (argc, argv, "d:nhpbir:o:c:")) != -1) {
+	while ((c = getopt (argc, argv, "o:d:r:c:fnbipRGBh")) != -1) {
 		switch (c)
 		{
-			case 'c':
-				which_cpu = atoi(optarg);
+			case 'o':
+				plugin = optarg;
 				break;
 			case 'd':
 				rotstep = optarg;
 				break;
-			case 'h':
-				print_help();
-				return 1;
+			case 'r':
+				rotdiv = optarg;
+				break;
+			case 'c':
+				which_cpu = atoi(optarg);
+				break;
+			case 'f':
+				font = 40;
+				break;
+			case 'n':
+				use_nice = 0;
+				break;
+			case 'b':
+				cube_color = 2;
+				break;
 			case 'i':
 				invert_speed = 1;
 				break;
 			case 'p':
 				show_load = 0;
 				break;
-			case 'b':
-				cube_color = 2;
+			case 'R':
+				red = 1;
 				break;
-			case 'r':
-				rotdiv = optarg;
+			case 'G':
+				green = 1;
 				break;
-			case 'o':
-				obj_filename = optarg;
-				break;
-			case 'n':
-				use_nice = 0;
+			case 'B':
+				blue = 1;
 				break;
+			case 'h':
+				print_help();
+				return 1;
 			case '?':
 					print_help();
 					return 1;
@@ -188,6 +222,8 @@ int main(int argc, char **argv)
 		}
 	}
 
+	set_plugin();
+
 	/*
 	 * Validate that wmcube can run on this system given the parameters,
 	 * then setup the statistics gathering subsystem.
@@ -225,35 +261,58 @@ int main(int argc, char **argv)
 
 	cpu_update /= screen_speed;
 
-	createXBMfromXPM(wmcube_mask_bits, wmcube_xpm, wmcube_mask_width, wmcube_mask_height);
-	openXwindow(argc, argv, wmcube_xpm, wmcube_mask_bits, wmcube_mask_width, wmcube_mask_height);
+	if ((red == 0) && (green == 0) && (blue == 0)) {
+		createXBMfromXPM(wmcube_mask_bits, wmcube_xpm, wmcube_mask_width, wmcube_mask_height);
+		openXwindow(argc, argv, wmcube_xpm, wmcube_mask_bits, wmcube_mask_width, wmcube_mask_height);
+	} else if ((red == 1) && (green == 0) && (blue == 0)) {
+		createXBMfromXPM(wmcube_mask_bits, wmcubered_xpm, wmcube_mask_width, wmcube_mask_height);
+		openXwindow(argc, argv, wmcubered_xpm, wmcube_mask_bits, wmcube_mask_width, wmcube_mask_height);
+	} else if ((red == 0) && (green == 1) && (blue == 0)) {
+		createXBMfromXPM(wmcube_mask_bits, wmcubegreen_xpm, wmcube_mask_width, wmcube_mask_height);
+		openXwindow(argc, argv, wmcubegreen_xpm, wmcube_mask_bits, wmcube_mask_width, wmcube_mask_height);
+	} else if ((red == 0) && (green == 0) && (blue == 1)) {
+		createXBMfromXPM(wmcube_mask_bits, wmcubeblue_xpm, wmcube_mask_width, wmcube_mask_height);
+		openXwindow(argc, argv, wmcubeblue_xpm, wmcube_mask_bits, wmcube_mask_width, wmcube_mask_height);
+	} else if ((red == 1) && (green == 1) && (blue == 0)) {
+		createXBMfromXPM(wmcube_mask_bits, wmcubeyellow_xpm, wmcube_mask_width, wmcube_mask_height);
+		openXwindow(argc, argv, wmcubeyellow_xpm, wmcube_mask_bits, wmcube_mask_width, wmcube_mask_height);
+	} else if ((red == 0) && (green == 1) && (blue == 1)) {
+		createXBMfromXPM(wmcube_mask_bits, wmcubecyan_xpm, wmcube_mask_width, wmcube_mask_height);
+		openXwindow(argc, argv, wmcubecyan_xpm, wmcube_mask_bits, wmcube_mask_width, wmcube_mask_height);
+	} else if ((red == 1) && (green == 0) && (blue == 1)) {
+		createXBMfromXPM(wmcube_mask_bits, wmcubemagenta_xpm, wmcube_mask_width, wmcube_mask_height);
+		openXwindow(argc, argv, wmcubemagenta_xpm, wmcube_mask_bits, wmcube_mask_width, wmcube_mask_height);
+	} else if ((red == 1) && (green == 1) && (blue == 1)) {
+		createXBMfromXPM(wmcube_mask_bits, wmcube_xpm, wmcube_mask_width, wmcube_mask_height);
+		openXwindow(argc, argv, wmcube_xpm, wmcube_mask_bits, wmcube_mask_width, wmcube_mask_height);
+	}
 
 	startup_seq();
 	
 	if (calc_cpu_total() == -1) die();
 
 	// index, left, top, right, bottom
-	AddMouseRegion(1,  45,  45, 58, 58);  // + Zoom In
+	AddMouseRegion(1,  45,  45, 58, 58);	  // + Zoom In
 	AddMouseRegion(5,  5, 45, 20, 58);	  // - Zoom Out
-	AddMouseRegion(3,  21, 45, 45, 58);	  //   Show cpu-load
- 	AddMouseRegion(2,  5, 5, 55, 45);	  // Everywhere else (almost) to change object
+	AddMouseRegion(3,  21, 45, 45, 58);	  // Show cpu-load
+	AddMouseRegion(2,  5, 5, 55, 45);	  // Everywhere else (almost) to change object
 
- 	/*
- 	 * Main loop begins here
- 	 */
+	/*
+	 * Main loop begins here
+	 */
 
 	while (1)
 	{
 		i = (i+rot_speed+rot_step) % 360;
 		
 		clearscr();
+
 		rotate(i,i,i);
-		
 		draw(cube_color);
 
 		if (show_load) {
 			BlitNum(cpu_usage,24,49);
-			BlitString("Z",38,49);
+			BlitString("%",38,49);
 		}
 
 		RedrawWindow();
@@ -277,6 +336,7 @@ int main(int argc, char **argv)
 		
 		while (XPending(display))
 		{
+
 			XNextEvent(display, &Event);
 			switch (Event.type)
 			{
@@ -337,16 +397,24 @@ int main(int argc, char **argv)
 	return 1;
 }
 
-
 //**** Graphics ***********************************
 //*************************************************
 
+void set_plugin()
+{
+	strcpy(obj_filename,plugin);
+	if (access (obj_filename, R_OK) == -1) {
+		strcpy(obj_filename,"/usr/share/wmcube/");
+		strcat(obj_filename,plugin);
+	}
+}
+
 void startup_seq() 
 {
 	char *tmp = malloc(32);
 	int oldzoff = 3600;
 
-	sprintf(tmp,"V%s",WMCUBE_VERSION);
+	sprintf(tmp,"%s",WMCUBE_VERSION);
 	
 	RedrawWindow();
 	BlitString("WMCUBE",13,22);
@@ -446,7 +514,7 @@ void triangle(int x1, int y1, int x2, int y2, int x3, int y3, int c) // Draws a
 		x2=x3;
 		x3=tmp1;
     }
-    
+
     if (y1!=y3)	k=((x1-x3) << 6) / (y1-y3);
     else k=(x1-x3) << 6;
 
@@ -513,37 +581,41 @@ void BlitString(char *name, int x, int y)
 		c = toupper(name[i]); 
       //printf("%c",c);
 		
+		if (c >= '0' && c<= '9')
+		{   // its a number or symbol
+			c -= '0';
+			if ( k > -2) copyXPMArea(c * 6, 64 + font, 6, 9, k, y);
+			k += 6;
+		} else
 		if (c >= 'A' && c <= 'Z')
 		{   // its a letter
 			c -= 'A';
-			if ( k > -2) copyXPMArea(c * 6, 74, 6, 8, k, y);
+			if ( k > -2) copyXPMArea(c * 6, 74 + font, 6, 9, k, y);
 			k += 6;
 		} else
-		if (c >= '0' && c<= ':')
-		{   // its a number or symbol
-			c -= '0';
-			if ( k > -2) copyXPMArea(c * 6, 64, 6, 8, k, y);
+		if (c == '.') {
+			if ( k > -2) copyXPMArea(60, 64 + font, 6, 9, k, y);
+			k += 6;
+		} else
+		if (c == '%') {
+			if ( k > -2) copyXPMArea(66, 64 + font, 6, 9, k, y);
 			k += 6;
 		} else
 		if (c == 246) {
-			if ( k > -2) copyXPMArea(0, 84, 6, 9, k, y);
+			if ( k > -2) copyXPMArea(0, 84 + font, 6, 9, k, y);
 			k += 6;
 		} else
 		if (c == 228) {
-			if ( k > -2) copyXPMArea(6, 84, 6, 9, k, y);
-			k += 6;
-		} else
-		if (c == 229) {
-			if ( k > -2) copyXPMArea(12, 84, 6, 9, k, y);
+			if ( k > -2) copyXPMArea(6, 84 + font, 6, 9, k, y);
 			k += 6;
 		} else
 		{   // its a blank or something else
-			if ( k > -2) copyXPMArea(73,64,6,8,k,y);
+			if ( k > -2) copyXPMArea(73, 64 + font, 6, 9, k, y);
 			k += 6;
 		}
 		if (k >= 58) break;
 	}
-	copyXPMArea(73,64,1,8,k,y);
+	copyXPMArea(73, 64 + font, 1, 9, k, y);
 	
 }
 
@@ -656,12 +728,13 @@ void rotate(int xang, int yang, int zang)
 				plane_color[i] = luminate(rmatrix[planes[i][0]], rmatrix[planes[i][1]], rmatrix[planes[i][2]]);
 
 	for (i = 0; i < nofcoords; i++) {	
-  		// Perspective correcting lines...
+		// Perspective correcting lines...
 		rmatrix[i][0] = (rmatrix[i][0] *256) / (2*rmatrix[i][2] - zoff) + xcenter;
 		rmatrix[i][1] = (rmatrix[i][1] *256) / (2*rmatrix[i][2] - zoff) + ycenter;
 	}
 }
 
+
 void sortz(int nofelements) {   // Insertion-sort the planes in increasing z-distance
 
 	int i, j, k;
@@ -682,7 +755,8 @@ void sortz(int nofelements) {   // Insertion-sort the planes in increasing z-dis
 			
 		while ((i > -1) && (temparr[i] > key)) {
 			temparr[i+1] = temparr[i];
-			zorder[i+1] = zorder[i--];
+			zorder[i+1] = zorder[i];
+			i--;
 		}
 
 		zorder[i+1] = k;										
@@ -795,11 +869,11 @@ void setupobj(char *filename)
 		planes[11][0] = 5; planes[11][1] = 7; planes[11][2] = 6;
 	}
 
- 	rmatrix = (float **)realloc(rmatrix,nofcoords*sizeof(float *)); mem_alloc_error(rmatrix);
- 	for (i = 0; i < nofcoords; i++) { 
- 		rmatrix[i] = (float *)malloc(3*sizeof(float));
- 		mem_alloc_error(rmatrix[i]);
- 	}
+	rmatrix = (float **)realloc(rmatrix,nofcoords*sizeof(float *)); mem_alloc_error(rmatrix);
+	for (i = 0; i < nofcoords; i++) { 
+		rmatrix[i] = (float *)malloc(3*sizeof(float));
+		mem_alloc_error(rmatrix[i]);
+	}
 
 	/*
 	 * Find the longest discance between all coordinates relative to the origin
@@ -830,38 +904,43 @@ void setupobj(char *filename)
 //*************************************************
 
 void print_help() {
-	printf("\nwmCube %s (%s)\n\n", WMCUBE_VERSION, REV_DATE);
+	printf("\nwmcube %s  (C) %s Cezary M. Kruk (%s)\n", WMCUBE_VERSION, REV_YEAR, REV_DATE);
+	printf("wmCube %s   (C) %s Robert Kling   (%s)\n\n", WMCUBE_OLDVERSION, OLDREV_YEAR, OLDREV_DATE);
 	
-#ifndef SOLARIS
-	printf("  -o <filename or directory>: load external 3d-object(s).\n\n");
-#else
-	printf("  -o <filename>: load external 3d-object.\n\n");
-#endif
-
-	printf("  -d x: rotate x degrees/step when the cpu is idle. (default 1)\n");
-	printf("  -r x: rotate 1 degree faster every x percent of cpu-usage. (default 25)\n");
+	printf("  Usage: wmcube [-o <filename>] [-drcfnbipRGBh]\n\n");
+	
+	printf("  -o <filename> : load external 3D object (plugin).\n\n");
+	printf("  -d x : rotate x degrees/step when the CPU is idle (default 1).\n");
+	printf("  -r x : rotate 1 degree faster every x percent of the CPU usage (default 25).\n");
 
 #ifdef LINUX
-	printf("  -c x: which cpu (0,1,2..) to monitor. (default average over all)\n");
-	printf("  -n  : exclude \"nice\" processes. (default OFF)\n");
+	printf("  -c x : which CPU (0, 1, 2...) to monitor (default average over all).\n");
+	printf("  -f   : use smooth font (default OFF).\n");
+	printf("  -n   : exclude \"nice\" processes (default OFF).\n");
 #endif
 
 #ifdef SOLARIS
-	printf("  -c x: which cpu (0,1,2..) to monitor. (default average over all)\n");
+	printf("  -c x : which CPU (0, 1, 2...) to monitor (default average over all).\n");
+	printf("  -f   : use smooth font (default OFF).\n");
 #endif
 
 #ifdef FREEBSD
-	printf("  -n  : exclude \"nice\" processes. (default OFF)\n");
+	printf("  -f   : use smooth font (default OFF).\n");
+	printf("  -n   : exclude \"nice\" processes (default OFF).\n");
 #endif
 
 #ifdef NETBSD
-	printf("  -n  : exclude \"nice\" processes. (default OFF)\n");
+	printf("  -f   : use smooth font (default OFF).\n");
+	printf("  -n   : exclude \"nice\" processes (default OFF).\n");
 #endif
 
-	printf("  -b  : draw the cube in a brighter color. (default OFF)\n");
-	printf("  -i  : invert cube speed. (default OFF)\n");
-	printf("  -p  : do not display cpu-load (default OFF)\n");
-	printf("  -h  : display this helptext.\n\n");	
+	printf("  -b   : draw the cube in a brighter color (default OFF).\n");
+	printf("  -i   : invert cube speed (default OFF).\n");
+	printf("  -p   : do not display the CPU load (default OFF).\n");
+	printf("  -R   : use red image.\n");
+	printf("  -G   : use green image.\n");
+	printf("  -B   : use blue image.\n");
+	printf("  -h   : display this help.\n\n");	
 }
 
 void die()
@@ -904,25 +983,24 @@ int loadobj(char *filename) {
 	
 	FILE *fp;
 	char tmp[64] = {""};
-	int i = 0, counter = 1;
+	int i = 0;
 
 	//printf("\nLoading file %s...",filename); fflush(stdout);
 	
 	if ((fp = fopen(filename,"rt")) == NULL) {
-		printf("\nERROR: wmCube object-file not found (%s).\n\n",filename);
+		printf("\nERROR: wmcube object-file not found (%s).\n\n",filename);
 		exit(0);
 	}
 
 	fscanf(fp,"%s",tmp);
 	
-	if (strcmp(tmp,"WMCUBE_COORDINATES") != 0) { 
+	if (strcmp(tmp,"WMCUBE_COORDINATES") != 0) {
 		printf("\nError in objectfile: it must start with WMCUBE_COORDINATES\n\n");
 		fclose(fp);
 		exit(0);
 	}
 
 	fscanf(fp,"%s",tmp);
-	counter = atoi(tmp);
 
 	while ((strcmp(tmp,"WMCUBE_LINES") != 0) && (strcmp(tmp,"WMCUBE_PLANES") != 0)) {		
 
@@ -956,7 +1034,8 @@ int loadobj(char *filename) {
 		while (1) {
 		
 			cline = (int *)realloc(cline,(i+2)*sizeof(int)); mem_alloc_error(cline);
-			fscanf(fp,"%d %d",&cline[i++],&cline[i++]);
+			fscanf(fp,"%d %d",&cline[i],&cline[i+1]);
+			i += 2;
 			//printf("\n%d %d",cline[i-2],cline[i-1]);
 			if (feof(fp)) break;
 		
@@ -1280,7 +1359,7 @@ int init_calc_cpu()
 
 int calc_cpu_total() {
         double avenrun[3];
-        
+
         (void) getloadavg(avenrun, sizeof(avenrun) / sizeof(avenrun[0]));
         return(((5.0*avenrun[0] + 0.5) > 50) ? 50 : (5.0*avenrun[0] + 0.5))*2;
 }
@@ -1305,7 +1384,7 @@ int calc_cpu_total ()
         size_t ssize;
         const int IDLE_TIME = 4;
         const int NICE_TIME = 1;
-        
+
         ssize = sizeof ( curr_cp_time );
         mib[0] = CTL_KERN;
         mib[1] = KERN_CP_TIME;
